defmodule Akkoma2org.ActivityStream.FileBackfill do
  def post_source(from) when is_binary(from), do: from
  def post_source(from) when is_map(from), do: Map.get(from, "content")

  @doc "Convert a map with a serialized AS2 Activity in it to our internal format"
  def munge_as2(as2_map) do
    obj = Map.get(as2_map, "object")
    type = as2_map |> Map.get("type")

    case type do
      "Create" ->
        post_src = obj |> Map.get("source") |> post_source

        %{
          external_id: obj |> Map.get("id"),
          action_type: type,
          actor: as2_map |> Map.get("actor"),
          dest: obj |> Map.get("to") |> Enum.at(0),
          published: obj |> Map.get("published"),
          type: obj |> Map.get("type"),
          summary: obj |> Map.get("summary"),
          content: obj |> Map.get("content"),
          post_source: post_src,
          full_action: Jason.encode!(as2_map)
        }

      "Announce" ->
        IO.warn("Announcements not supported")
        %{}
    end
  end

  def ingest_one_as2(item) do
    case item
         |> Akkoma2org.ActivityStream.munge_as2()
         |> Akkoma2org.ActivityStream.create_action() do
      {:ok, _result} -> IO.puts("inserted")
      {:error, _} -> IO.warn("failed")
    end
  end

  def ingest_file(path) do
    with {:ok, handle} <- :zip.zip_open(to_charlist(path), [:memory]),
         {:ok, {'outbox.json', res}} = :zip.zip_get("outbox.json", handle) do
      res
      |> Jason.decode!()
      |> Map.get("orderedItems")
      |> Enum.map(&ingest_one_as2(&1))

      :zip.zip_close(handle)
    end
  end
end
