defmodule Akkoma2orgWeb.ActionLive.FormComponent do
  use Akkoma2orgWeb, :live_component

  alias Akkoma2org.ActivityStream

  @impl true
  def render(assigns) do
    ~H"""
    <div>
      <.header>
        <%= @title %>
        <:subtitle>Use this form to manage action records in your database.</:subtitle>
      </.header>

      <.simple_form
        for={@form}
        id="action-form"
        phx-target={@myself}
        phx-change="validate"
        phx-submit="save"
      >
        <.input field={@form[:action_type]} type="text" label="Actiont ype" />
        <.input field={@form[:actor]} type="text" label="Actor" />
        <.input field={@form[:content]} type="text" label="Content" />
        <.input field={@form[:dest]} type="text" label="Dest" />
        <.input field={@form[:external_id]} type="text" label="External" />
        <.input field={@form[:published]} type="text" label="Published" />
        <.input field={@form[:post_source]} type="text" label="Post source" />
        <.input field={@form[:summary]} type="text" label="Summary" />
        <.input field={@form[:type]} type="text" label="Type" />
        <.input field={@form[:full_action]} type="text" label="Full action" />
        <.input field={@form[:rank]} type="number" label="Rank" step="any" />
        <:actions>
          <.button phx-disable-with="Saving...">Save Action</.button>
        </:actions>
      </.simple_form>
    </div>
    """
  end

  @impl true
  def update(%{action: action} = assigns, socket) do
    changeset = ActivityStream.change_action(action)

    {:ok,
     socket
     |> assign(assigns)
     |> assign_form(changeset)}
  end

  @impl true
  def handle_event("validate", %{"action" => action_params}, socket) do
    changeset =
      socket.assigns.action
      |> ActivityStream.change_action(action_params)
      |> Map.put(:action, :validate)

    {:noreply, assign_form(socket, changeset)}
  end

  def handle_event("save", %{"action" => action_params}, socket) do
    save_action(socket, socket.assigns.action, action_params)
  end

  defp save_action(socket, :edit, action_params) do
    case ActivityStream.update_action(socket.assigns.action, action_params) do
      {:ok, action} ->
        notify_parent({:saved, action})

        {:noreply,
         socket
         |> put_flash(:info, "Action updated successfully")
         |> push_patch(to: socket.assigns.patch)}

      {:error, %Ecto.Changeset{} = changeset} ->
        {:noreply, assign_form(socket, changeset)}
    end
  end

  defp save_action(socket, :new, action_params) do
    case ActivityStream.create_action(action_params) do
      {:ok, action} ->
        notify_parent({:saved, action})

        {:noreply,
         socket
         |> put_flash(:info, "Action created successfully")
         |> push_patch(to: socket.assigns.patch)}

      {:error, %Ecto.Changeset{} = changeset} ->
        {:noreply, assign_form(socket, changeset)}
    end
  end

  defp assign_form(socket, %Ecto.Changeset{} = changeset) do
    assign(socket, :form, to_form(changeset))
  end

  defp notify_parent(msg), do: send(self(), {__MODULE__, msg})
end
