defmodule Akkoma2orgWeb.Router do
  use Akkoma2orgWeb, :router

  pipeline :browser do
    plug :accepts, ["html"]
    plug :fetch_session
    plug :fetch_live_flash
    plug :put_root_layout, html: {Akkoma2orgWeb.Layouts, :root}
    plug :protect_from_forgery
    plug :put_secure_browser_headers
  end

  pipeline :api do
    plug :accepts, ["json"]
  end

  scope "/", Akkoma2orgWeb do
    pipe_through :browser

    get "/", PageController, :home

    live "/actions", ActionLive.Index, :index
    live "/actions/new", ActionLive.Index, :new
    live "/actions/:id/edit", ActionLive.Index, :edit

    live "/actions/:id", ActionLive.Show, :show
    live "/actions/:id/show/edit", ActionLive.Show, :edit
  end

  # Other scopes may use custom stacks.
  # scope "/api", Akkoma2orgWeb do
  #   pipe_through :api
  # end

  # Enable LiveDashboard and Swoosh mailbox preview in development
  if Application.compile_env(:akkoma2org, :dev_routes) do
    # If you want to use the LiveDashboard in production, you should put
    # it behind authentication and allow only admins to access it.
    # If your application does not have an admins-only section yet,
    # you can use Plug.BasicAuth to set up some basic authentication
    # as long as you are also using SSL (which you should anyway).
    import Phoenix.LiveDashboard.Router

    scope "/dev" do
      pipe_through :browser

      live_dashboard "/dashboard", metrics: Akkoma2orgWeb.Telemetry
      forward "/mailbox", Plug.Swoosh.MailboxPreview
    end
  end
end
