defmodule Akkoma2orgWeb.ActionLiveTest do
  use Akkoma2orgWeb.ConnCase

  import Phoenix.LiveViewTest
  import Akkoma2org.ActivityStreamFixtures

  @create_attrs %{action_type: "some action_type", actor: "some actor", content: "some content", dest: "some dest", external_id: "some external_id", full_action: "some full_action", post_source: "some post_source", published: "some published", rank: 120.5, summary: "some summary", type: "some type"}
  @update_attrs %{action_type: "some updated action_type", actor: "some updated actor", content: "some updated content", dest: "some updated dest", external_id: "some updated external_id", full_action: "some updated full_action", post_source: "some updated post_source", published: "some updated published", rank: 456.7, summary: "some updated summary", type: "some updated type"}
  @invalid_attrs %{action_type: nil, actor: nil, content: nil, dest: nil, external_id: nil, full_action: nil, post_source: nil, published: nil, rank: nil, summary: nil, type: nil}

  defp create_action(_) do
    action = action_fixture()
    %{action: action}
  end

  describe "Index" do
    setup [:create_action]

    test "lists all actions", %{conn: conn, action: action} do
      {:ok, _index_live, html} = live(conn, ~p"/actions")

      assert html =~ "Listing Actions"
      assert html =~ action.action_type
    end

    test "saves new action", %{conn: conn} do
      {:ok, index_live, _html} = live(conn, ~p"/actions")

      assert index_live |> element("a", "New Action") |> render_click() =~
               "New Action"

      assert_patch(index_live, ~p"/actions/new")

      assert index_live
             |> form("#action-form", action: @invalid_attrs)
             |> render_change() =~ "can&#39;t be blank"

      assert index_live
             |> form("#action-form", action: @create_attrs)
             |> render_submit()

      assert_patch(index_live, ~p"/actions")

      html = render(index_live)
      assert html =~ "Action created successfully"
      assert html =~ "some action_type"
    end

    test "updates action in listing", %{conn: conn, action: action} do
      {:ok, index_live, _html} = live(conn, ~p"/actions")

      assert index_live |> element("#actions-#{action.id} a", "Edit") |> render_click() =~
               "Edit Action"

      assert_patch(index_live, ~p"/actions/#{action}/edit")

      assert index_live
             |> form("#action-form", action: @invalid_attrs)
             |> render_change() =~ "can&#39;t be blank"

      assert index_live
             |> form("#action-form", action: @update_attrs)
             |> render_submit()

      assert_patch(index_live, ~p"/actions")

      html = render(index_live)
      assert html =~ "Action updated successfully"
      assert html =~ "some updated action_type"
    end

    test "deletes action in listing", %{conn: conn, action: action} do
      {:ok, index_live, _html} = live(conn, ~p"/actions")

      assert index_live |> element("#actions-#{action.id} a", "Delete") |> render_click()
      refute has_element?(index_live, "#actions-#{action.id}")
    end
  end

  describe "Show" do
    setup [:create_action]

    test "displays action", %{conn: conn, action: action} do
      {:ok, _show_live, html} = live(conn, ~p"/actions/#{action}")

      assert html =~ "Show Action"
      assert html =~ action.action_type
    end

    test "updates action within modal", %{conn: conn, action: action} do
      {:ok, show_live, _html} = live(conn, ~p"/actions/#{action}")

      assert show_live |> element("a", "Edit") |> render_click() =~
               "Edit Action"

      assert_patch(show_live, ~p"/actions/#{action}/show/edit")

      assert show_live
             |> form("#action-form", action: @invalid_attrs)
             |> render_change() =~ "can&#39;t be blank"

      assert show_live
             |> form("#action-form", action: @update_attrs)
             |> render_submit()

      assert_patch(show_live, ~p"/actions/#{action}")

      html = render(show_live)
      assert html =~ "Action updated successfully"
      assert html =~ "some updated action_type"
    end
  end
end
